@@ -944,7 +944,8 @@ def R_matrix(N1,N2,N3,tot_atoms_uc,R0, H):
     Scells = get_Scells(H, R0)
     
     
-    R_matrix = np.zeros((tot_atoms_uc*3,N*3))
+    R_matrix_old = np.zeros((tot_atoms_uc*3,N*3))
+    R_matrix_new = np.zeros((N*3,tot_atoms_uc*3))
     count = 0
     for i in range(tot_atoms_uc):
         atom_origin = R0[i*N1N2N3]
@@ -956,14 +957,47 @@ def R_matrix(N1,N2,N3,tot_atoms_uc,R0, H):
             found_min_dist_0 = found_min_dist[0]
             row.append((Scells[found_min_dist_0,j,:] - atom_origin).flatten())
         row = np.array(row)
-        
         repeated_row = np.tile(row.flatten(),(3,1))
-        R_matrix[count:count+3,:] = repeated_row
+        R_matrix_old[count:count+3,:] = repeated_row
+        # R_matrix_new[count_new] = 
         count =  count + 3
+        R_matrix_new[:,i*3:i*3+3] = np.repeat(row.flatten().reshape(N,3),3,0)
         
-    return R_matrix#*0.529177249
+    return R_matrix_old, R_matrix_new# *0.529177249
+
+
 
 def get_D(k, K,N1,N2,N3,R0,masses,tot_atoms_uc, H):
+    """
+    New version of get_D, two times faster than get_D_old. Highly vectorised.
+    
+    builds the dynamical matrix
+    expects forces in 1/amu * ev / A**2
+    returns Thz**2
+    """
+    N1N2N3 = N1 * N2 * N3
+    N = N1N2N3*tot_atoms_uc
+    
+    R, Rnew = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
+    #prepare kdotr
+    exp_coeff = np.zeros((tot_atoms_uc*3,N*3), dtype=complex)
+    for i in range(tot_atoms_uc):
+        R = Rnew[:,i*3:i*3+3]
+        kdotR = np.exp(1j*np.dot(R,k))
+        exp_coeff[i*3,:] = kdotR
+        exp_coeff[i*3+1,:] = kdotR
+        exp_coeff[i*3+2,:] = kdotR
+    
+    mass_coeff = 1/np.sqrt(np.outer(masses, np.repeat(masses, N1N2N3)))
+    Dall = mass_coeff*K*exp_coeff
+    Da = np.reshape(Dall, (tot_atoms_uc*3,N,3))
+    D = np.zeros((tot_atoms_uc*3, tot_atoms_uc*3), dtype=complex)
+    for j in range(tot_atoms_uc):
+        D[:, j*3:j*3+3] = np.sum(Da[:,j*N1N2N3:(j+1)*N1N2N3,:], axis=1)
+        
+    return (D+D.conj().transpose())/2*0.964*10**(4)
+
+def get_D_old(k, K,N1,N2,N3,R0,masses,tot_atoms_uc, H):
     """
     builds the dynamical matrix
     expects forces in 1/amu * ev / A**2
@@ -972,7 +1006,7 @@ def get_D(k, K,N1,N2,N3,R0,masses,tot_atoms_uc, H):
     N1N2N3 = N1 * N2 * N3
     N = N1N2N3*tot_atoms_uc
     
-    R = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
+    R, Rnew = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
     #prepare kdotr
     kdotr = np.zeros((tot_atoms_uc*3,N*3))
     for i in range(tot_atoms_uc*3):
